clear all;
%close all;

global T N l1 l2 m1 m2 Icm1 Icm2 g c1 c2 theta10 theta20 theta1dot0 ...
       theta2dot0 tau10 tau20 theta1f theta2f theta1dotf theta2dotf tau1f tau2f...
       B11 B12 B21 B22  

% Gravitational force
g=9.81;

% Time span (N number of discretization intervals, which is only used only 
% for showing results).
N=100; 
T=0.5;

% Arm parameters (taken from a Todorov's paper)
m1   = 1.4;    % segment mass
m2   = 1.1;
l1   = 0.3;    % segment length
l2   = 0.33;
c1   = 0.11;   % segment center of mass
c2   = 0.16;
Icm1   = 0.025;  % segment moment of inertia
Icm2   = 0.045;
B11  = 0.5;    % joint friction
B22  = 0.5;  
B12  = 0.1; 
B21  = 0.1; 

% Initial state [q0, 0, g(q0)]
theta10= 40*pi/180; 
theta20= 75*pi/180;
theta1dot0=0;
theta2dot0=0;
tau10= m1*c1*g*cos(theta10)+m2*g*(c2*cos(theta10+theta20)+l1*cos(theta10));
tau20= m2*g*c2*cos(theta10+theta20);


% Final state [qf, 0, g(qf)]
theta1f= 90*pi/180;
theta2f= 30*pi/180;
theta1dotf=0;
theta2dotf=0;
tau1f= m1*c1*g*cos(theta1f)+m2*g*(c2*cos(theta1f+theta2f)+l1*cos(theta1f));
tau2f= m2*g*c2*cos(theta1f+theta2f);

disp('%%%%%%%%%% TORQUE CHANGE %%%%%%%%%%');

% Using a good IniGuess for the adjoint vector speeds up and helps for
% convergence... This can be obtained from the solution obtained for the
% linearized system, or from a numerical solution using GPOPS or similar
% discretized method solving NLPs.

%Shooting method for computing the intial costate that hits the disired
%final state
IniGuess=randn(1,6);
Result=TorqueChangeF(IniGuess); 

%% Plotting the reults (cartesian movement and velocity profile)
subplot (121)
xTraj = l1*cos(Result.State(:,1)) + l2*cos(Result.State(:,1) + Result.State(:,2));
yTraj = l1*sin(Result.State(:,1)) + l2*sin(Result.State(:,1) + Result.State(:,2));
plot(xTraj, yTraj, '.')
hold on
nSteps = 10;
l1ext = [l1*cos(Result.State(1:nSteps:end,1)), l1*sin(Result.State(1:nSteps:end,1,1))];
l2ext = l1ext + [l2*cos(Result.State(1:nSteps:end,1) + Result.State(1:nSteps:end,2)), l2*sin(Result.State(1:nSteps:end,1,1) + Result.State(1:nSteps:end,2))];
for k = 1: length(l1ext(:,1))
    plot([0, l1ext(k,1), l2ext(k,1)], [0, l1ext(k,2), l2ext(k,2)], 'k', 'LineWidth', 3);
end
grid on
axis equal
title('Cartesian Movement')
xlabel('x[m]')
ylabel('y[m]')

subplot (122)
plot(Result.Time(1:end-1), sqrt(diff(xTraj).^2 + diff(yTraj).^2)./diff(Result.Time), 'LineWidth', 2)
hold on
grid on
title('Velocity profile')
ylabel('Velocity[m/s]')
