function Results=TorqueChangeF(IniAdjoint)

global T N l1 l2 theta10 theta20 theta1dot0 theta2dot0 ...
       tau10 tau20  m1 m2 g c1 c2 Icm1 Icm2 B11 B12 B21 B22 
          
%Setting the properties of the numerical optimization
options = optimset('GradObj','off','Display','iter', ...
                   'TolX',1e-6,'TolCon',1e-6,'TolFun',1e-6,'maxiter',1000, ...
                   'MaxFunEvals',10000,'NonlEqnAlgorithm','lm','Algorithm','trust-region-reflective'); % 
%Solving the shooting optimization
[res,fval,exitflag,output,jacobian]=fsolve(@(z) TorqueChangeOptimFunF(z),IniAdjoint,options);

if fval > 1e-10
    disp('ERROR: could not find a good solution for the shooting method!')
    disp('ERROR: computed solution is not correct!')
end

%% COMPUTE THE OPTIMAL SOLUTION: just recomputing the given solution

%retrieving the computed solution
iniAdjoint=res;
IniState=[ theta10 theta20 theta1dot0 theta2dot0 tau10 tau20 ...
           iniAdjoint ];
times=linspace(0,T,N)';
options = odeset('RelTol',1e-4,'AbsTol',1e-4);

%simularting the state-costate dynamic equation in order to compute the
%state-costate evolution associated to the computed solution
[tmp,Sol] = ode45(@(t,x) TorqueChangeSystemDiff(t,x), times, IniState, options);

%state evolution
theta1=Sol(:,1);
theta2=Sol(:,2);
thetadot1=Sol(:,3);
thetadot2=Sol(:,4);
tau1=Sol(:,5);
tau2=Sol(:,6);

%costate evolution
p1=Sol(:,7);
p2=Sol(:,8);
p3=Sol(:,9);
p4=Sol(:,10);
p5=Sol(:,11);
p6=Sol(:,12);

%rearranging results in a proper structure
Results.CoState=[p1 p2 p3 p4 p5 p6];
Results.State = [theta1 theta2 thetadot1 thetadot2 tau1 tau2];
Results.Time = times;

