clc
clear all

syms q1 q2 dq1 dq2 h real 
syms tau1 tau2 u1 u2 p1 p2 p3 p4 p5 p6 Minv11 Minv12 Minv22 B11 B12 B21 B22 real

Minv11 = sym('Minv11(q2)');
Minv12 = sym('Minv12(q2)');
Minv22 = sym('Minv22(q2)');
h      = sym('h(q2)');
g1     = sym('g1(q1, q2)');
g2     = sym('g2(q1, q2)');

C = [-h*dq2,      -h*(dq1+dq2);
      h*dq1,        0];

Minv11 = sym('Minv11(q2)');
Minv12 = sym('Minv12(q2)');
Minv22 = sym('Minv22(q2)');

u = [u1; u2];
tau = [tau1; tau2];
H = 1/2*u'*u + ...
    [p1, p2] * [dq1; dq2] + ...
    [p3, p4] * [Minv11, Minv12; Minv12, Minv22] * (tau - C * [dq1; dq2] - [B11, B12; B21, B22] * [dq1; dq2] - [g1; g2]) + ...
    [p5, p6] * u;

H = simple(H);

%State dynamic equation
dx1 = simple(diff(H, p1));
dx2 = simple(diff(H, p2));
dx3 = simple(diff(H, p3));
dx4 = simple(diff(H, p4));
dx5 = simple(diff(H, p5));
dx6 = simple(diff(H, p6));
dx = [dx1; dx2; dx3; dx4; dx5; dx6];

%Costate dynamic equation
dp1 = simple(diff(-H, q1));
dp2 = simple(diff(-H, q2));
dp3 = simple(diff(-H, dq1));
dp4 = simple(diff(-H, dq2));
dp5 = simple(diff(-H, tau1));
dp6 = simple(diff(-H, tau2));
dp = [dp1; dp2; dp3; dp4; dp5; dp6];

syms alpha beta delta m1 m2 l1 l2 c1 c2 Icm1 Icm2 g g1 g2 real 
 
M = [alpha + 2*beta*cos(q2),   delta + beta*cos(q2);
     delta + beta*cos(q2),     delta];
 
alpha = Icm1 + Icm2 + m1*c1^2 + m2*(l1^2+c2^2);
beta  = m2*l1*c2;
delta = Icm2 + m2*c2^2;

h = beta*sin(q2);

g1 = m1*g*c1*cos(q1) + m2*g*(l1*cos(q1) + c2*cos(q1+q2));
g2 = m2*g*c2*cos(q1+q2);

%%%
Minv = simple(inv(M));
Minv11 = Minv(1,1);
Minv12 = Minv(1,2);
Minv22 = Minv(2,2);

dg1_dq1     = diff(g1, q1);
dg1_dq2     = diff(g1, q2);
dh_dq2      = diff(h, q2);
dg2_dq1     = diff(g2, q1);
dg2_dq2     = diff(g2, q2);
dMinv11_dq2 = diff(Minv11, q2);
dMinv12_dq2 = diff(Minv12, q2);
dMinv22_dq2 = diff(Minv22, q2);

%% Conversion to strings %%%%%%%%%%%%
%M%
disp(sprintf('alpha = %s;', char(alpha)))
disp(sprintf('beta  = %s;', char(beta)))
disp(sprintf('delta = %s;\n', char(delta)))
Minv11Str = char(Minv11);
disp(sprintf('Minv11 = %s;', Minv11Str))
Minv12Str = char(Minv12);
disp(sprintf('Minv12 = %s;', Minv12Str))
Minv22Str = char(Minv22);
disp(sprintf('Minv22 = %s;\n', Minv22Str))

%C%
disp(sprintf('h = %s;', char(h)))
disp(sprintf('C11 = %s;', replaceSymbDepend(char(C(1,1)))))
disp(sprintf('C12 = %s;', replaceSymbDepend(char(C(1,2)))))
disp(sprintf('C21 = %s;', replaceSymbDepend(char(C(2,1)))))
disp(sprintf('C22 = %s;\n', replaceSymbDepend(char(C(2,2)))))

%g%
disp(sprintf('g1 = %s;', char(g1)))
disp(sprintf('g2 = %s;\n', char(g2)))

disp(sprintf('dg1_dq1 = %s;', char(diff(g1, q1))))
disp(sprintf('dg1_dq2 = %s;', char(diff(g1, q2))))
disp(sprintf('dg2_dq1 = %s;', char(diff(g2, q1))))
disp(sprintf('dg2_dq2 = %s;\n', char(diff(g2, q2))))

disp(sprintf('dh_dq2 = %s;\n', char(diff(h, q2))))

disp(sprintf('dMinv11_dq2 = %s;', char(diff(Minv11, q2))))
disp(sprintf('dMinv12_dq2 = %s;', char(diff(Minv12, q2))))
disp(sprintf('dMinv22_dq2 = %s;\n', char(diff(Minv22, q2))))

%optimal input
disp(sprintf('u1 = -p5;'));
disp(sprintf('u2 = -p6;\n'));

%state
dx1Str = char(dx1);
dx1Str = replaceSymbDepend(dx1Str);
disp(sprintf('dx1 = %s;', dx1Str))
dx2Str = char(dx2);
dx2Str = replaceSymbDepend(dx2Str);
disp(sprintf('dx2 = %s;', dx2Str))
dx3Str = char(dx3);
dx3Str = replaceSymbDepend(dx3Str);
disp(sprintf('dx3 = %s;', dx3Str))
dx4Str = char(dx4);
dx4Str = replaceSymbDepend(dx4Str);
disp(sprintf('dx4 = %s;', dx4Str))
dx5Str = char(dx5);
dx5Str = replaceSymbDepend(dx5Str);
disp(sprintf('dx5 = %s;', dx5Str))
dx6Str = char(dx6);
dx6Str = replaceSymbDepend(dx6Str);
disp(sprintf('dx6 = %s;\n', dx6Str))

%costate
dp1Str = char(dp1);
dp1Str = replaceSymbDepend(dp1Str);
disp(sprintf('dp1 = %s;', dp1Str))
dp2Str = char(dp2);
dp2Str = replaceSymbDepend(dp2Str);
disp(sprintf('dp2 = %s;', dp2Str))
dp3Str = char(dp3);
dp3Str = replaceSymbDepend(dp3Str);
disp(sprintf('dp3 = %s;', dp3Str))
dp4Str = char(dp4);
dp4Str = replaceSymbDepend(dp4Str);
disp(sprintf('dp4 = %s;', dp4Str))
dp5Str = char(dp5);
dp5Str = replaceSymbDepend(dp5Str);
disp(sprintf('dp5 = %s;', dp5Str))
dp6Str = char(dp6);
dp6Str = replaceSymbDepend(dp6Str);
disp(sprintf('dp6 = %s;\n', dp6Str))
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%





